package com.drugme.ms.pillbox;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.support.v7.app.AppCompatActivity;
import android.telephony.gsm.SmsManager;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.GridView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.drugme.ms.pillbox.alarme.AlarmReceiver;
import com.drugme.ms.pillbox.gsm.AlerteReceiver;
import com.drugme.ms.pillbox.connectivity.Bluetooth;
import com.drugme.ms.pillbox.connectivity.HexAscii;
import com.drugme.ms.pillbox.util.SharedPreference;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

public class MainActivity extends AppCompatActivity implements BluetoothAdapter.LeScanCallback {

    final private static String TAG = "MainActivity";

    // OBJET SUIVI HEBDO
    private MyAdapterImage myAdapterImage;
    public Integer[] imgDaily = {R.drawable.point_noir, R.drawable.point_rouge, R.drawable.point_vert, R.drawable.point_jaune};

    // ALARME
    private PendingIntent pendingIntent;

    // OBJET ALARME
    private Button stopAlarm;
    private Boolean onAlarme = false;
    private Boolean frontMontant = true;
    private Long debutAlarme;
    private Long delay;

    // BLUETOOTH
    final private static int STATE_BLUETOOTH_OFF = 1;
    final private static int STATE_DISCONNECTED = 2;
    final private static int STATE_CONNECTING = 3;
    final private static int STATE_CONNECTED = 4;
    private int state;
    private boolean scanStarted;
    private boolean scanning;

    // OBJETS BLUETOOTH
    private BluetoothAdapter bluetoothAdapter;
    private BluetoothDevice bluetoothDevice;
    private RFduinoService rfduinoService;
    private Button enableBluetoothButton;
    private TextView scanStatusText;
    private Button scanButton;
    private TextView deviceInfoText;
    private TextView connectionStatusText;
    private Button connectButton;
    private LinearLayout dataLayout;

    // GSM
    private SharedPreference sharedPreference;
    private String text;
    private String sendNumber;
    private String message = "Votre patient n'a pas pris son médicament.";

    // OBJETS GSM
    private EditText number;
    private Button recordNumber;
    private TextView recordedNumber;

    // Données
    private String ascii;

    // Date
    TextView Date;

    private final BroadcastReceiver bluetoothStateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int state = intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, 0);
            if (state == BluetoothAdapter.STATE_ON) {
                upgradeState(STATE_DISCONNECTED);
            } else if (state == BluetoothAdapter.STATE_OFF) {
                downgradeState(STATE_BLUETOOTH_OFF);
            }
        }
    };

    private final BroadcastReceiver scanModeReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            scanning = (bluetoothAdapter.getScanMode() != BluetoothAdapter.SCAN_MODE_NONE);
            scanStarted &= scanning;
            updateUi();
        }
    };

    private final ServiceConnection rfduinoServiceConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            rfduinoService = ((RFduinoService.LocalBinder) service).getService();
            if (rfduinoService.initialize()) {
                if (rfduinoService.connect(bluetoothDevice.getAddress())) {
                    upgradeState(STATE_CONNECTING);
                }
            }
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            rfduinoService = null;
            downgradeState(STATE_DISCONNECTED);
        }
    };

    private final BroadcastReceiver rfduinoReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            final String action = intent.getAction();
            if (RFduinoService.ACTION_CONNECTED.equals(action)) {
                upgradeState(STATE_CONNECTED);
                Log.d(TAG, "rfduinoReceiver State connected");
            } else if (RFduinoService.ACTION_DISCONNECTED.equals(action)) {
                downgradeState(STATE_DISCONNECTED);
            } else if (RFduinoService.ACTION_DATA_AVAILABLE.equals(action)) {
                byte[] data = intent.getByteArrayExtra(RFduinoService.EXTRA_DATA);
                ascii = HexAscii.bytesToAsciiMaybe(data);
                Log.d(TAG, ascii);

                if (ascii.equals("alarme")){
                    boolean check = checkStateAlarm();

                    if (!check){
                        onAlarme = false;
                    }

                    if(!onAlarme){
                        startAlarme();
                    }
                }
                else {
                MainActivity.this.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                         String[] object = ascii.split(":");

                            if (object[0].equals("Date")){

                            }
                            else if (object[0].equals("Etat")){
                                Integer a, b, c, d;
                                String[] tokens = object[1].split(";");
                                a = Integer.parseInt(tokens[0]);
                                b = Integer.parseInt(tokens[1]);
                                c = Integer.parseInt(tokens[2]);
                                d = Integer.parseInt(tokens[3]);
                                receptionEtatPilulier(a, b, c, d);
                            }
                            else {
                                Toast.makeText(MainActivity.this, "Error ", Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
                }
            }
        }
    };

    private void receptionEtatPilulier(int matin, int midi, int soir, int nuit) {
        myAdapterImage.imgDaily[0]=selectImgColorTime(matin);
        myAdapterImage.imgDaily[1]=selectImgColorTime(midi);
        myAdapterImage.imgDaily[2]=selectImgColorTime(soir);
        myAdapterImage.imgDaily[3]=selectImgColorTime(nuit);
        myAdapterImage.notifyDataSetChanged();
    }

    private Integer selectImgColorTime (int time) {
        Integer img;

        if (time==1)
            img=R.drawable.point_jaune;
        else if (time==2)
            img=R.drawable.point_vert;
        else if (time==3)
            img=R.drawable.point_rouge;
        else if (time==4)
            img=R.drawable.point_noir;
        else
            img=0;

        return img;
    }

    private void startAlarme() {
        onAlarme = true;

        Toast.makeText(MainActivity.this, "Start Alarm", Toast.LENGTH_LONG).show();

        AlarmManager am = (AlarmManager) getSystemService(ALARM_SERVICE);

        Intent Myintent1 = new Intent(getBaseContext(), AlarmReceiver.class);
        Intent Myintent2 = new Intent(getBaseContext(), AlerteReceiver.class);

        // Récupération du numéro
        recordedNumber = (TextView) findViewById(R.id.recordedNumber);
        sendNumber = recordedNumber.getText().toString();


        Myintent2.putExtra("number", sendNumber);

        // Prepare intent to launch notification
        PendingIntent pendingIntent1 = PendingIntent.getBroadcast(MainActivity.this, 1, Myintent1, PendingIntent.FLAG_UPDATE_CURRENT);
        PendingIntent pendingIntent2 = PendingIntent.getBroadcast(MainActivity.this, 2, Myintent2, PendingIntent.FLAG_UPDATE_CURRENT);

        // The alarm manager is an android system service
        am.setRepeating(AlarmManager.RTC, System.currentTimeMillis(), 3000, pendingIntent1); //3 minutes : 180000
        am.set(AlarmManager.RTC, System.currentTimeMillis() + 20000, pendingIntent2); // 2 heures : 7200000
    }

    private void stopAlarme() {
        boolean checkAlarm = checkStateAlarm();
        //Toast.makeText(MainActivity.this, "State alarm MainActivity: "+checkAlarm+" onAlarme: "+onAlarme, Toast.LENGTH_SHORT).show();
        // Prepare intent to launch notification
        if (checkAlarm){
            onAlarme=false;
            Toast.makeText(MainActivity.this, "Stop Alarm", Toast.LENGTH_LONG).show();
            Intent Myintent = new Intent(getBaseContext(), AlarmReceiver.class);
            PendingIntent pendingIntent = PendingIntent.getBroadcast(MainActivity.this, 1, Myintent, PendingIntent.FLAG_UPDATE_CURRENT);
            AlarmManager am = (AlarmManager) getSystemService(ALARM_SERVICE);
            am.cancel(pendingIntent);
            pendingIntent.cancel();
        }
    }

    private boolean checkStateAlarm (){
        Intent Myintent = new Intent(getApplicationContext(), AlarmReceiver.class);
        return (PendingIntent.getBroadcast(getApplicationContext(), 1,Myintent, PendingIntent.FLAG_NO_CREATE) != null);
    }

    private void sendMessage(String sendNumber){

        String message = "Votre patient n'a pas pris son médicament.";

        // Si le numéro est égale à 10 alors on envoi le message d'alerte
        if (sendNumber.length() == 10) {
            SmsManager.getDefault().sendTextMessage(sendNumber, null, message, null, null);
            Toast.makeText(getApplicationContext(), "Un message de prévention a été envoyé.", Toast.LENGTH_SHORT).show();
        } else {
            Toast.makeText(getApplicationContext(), "Veillez indiquer un numéro de téléphone à 10 chiffres.", Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // Bluetooth
        enableBluetoothButton = (Button) findViewById(R.id.enableBluetooth);
        enableBluetoothButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                enableBluetoothButton.setEnabled(false);
                enableBluetoothButton.setText(
                        bluetoothAdapter.enable() ? "Enabling bluetooth..." : "Enable failed!");
            }
        });

        // Find Device
        scanStatusText = (TextView) findViewById(R.id.scanStatus);

        scanButton = (Button) findViewById(R.id.scan);
        scanButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                scanStarted = true;
                bluetoothAdapter.startLeScan(
                        new UUID[]{RFduinoService.UUID_SERVICE},
                        MainActivity.this);
            }
        });

        // Device Info
        deviceInfoText = (TextView) findViewById(R.id.deviceInfo);

        // Connect Device
        connectionStatusText = (TextView) findViewById(R.id.connectionStatus);

        connectButton = (Button) findViewById(R.id.connect);
        connectButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                v.setEnabled(false);
                connectionStatusText.setText("Connecting...");
                Intent rfduinoIntent = new Intent(MainActivity.this, RFduinoService.class);
                bindService(rfduinoIntent, rfduinoServiceConnection, BIND_AUTO_CREATE);
            }
        });

        dataLayout = (LinearLayout) findViewById(R.id.dataLayout);

        sharedPreference = new SharedPreference();

        // Récupération du numéro enregistré
        recordedNumber = (TextView) findViewById(R.id.recordedNumber);
        text = sharedPreference.getValue(MainActivity.this);
        recordedNumber.setText(text);

        // Rédaction du numéro
        number = (EditText) findViewById(R.id.number);
        number.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    // Hides the soft keyboard
                }
            }
        });

        // Enregistrement du numéro
        recordNumber = (Button) findViewById(R.id.recordNumber);
        recordNumber.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                text = number.getText().toString();
                text = number.getText().toString();

                // Hides the soft keyboard
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(number.getWindowToken(), 0);

                // Save the text in SharedPreference only if is a phone number
                if (text.length() == 10) {
                    sharedPreference.save(MainActivity.this, text);
                    recordedNumber.setText(text);
                    Toast.makeText(MainActivity.this,
                            getResources().getString(R.string.saved),
                            Toast.LENGTH_LONG).show();
                } else {
                    Toast.makeText(MainActivity.this,
                            getResources().getString(R.string.errorSave),
                            Toast.LENGTH_LONG).show();
                }
            }
        });

        GridView gridviewText = (GridView) findViewById(R.id.gridviewText);
        gridviewText.setAdapter(new MyAdapterText(this));

        GridView gridviewImage = (GridView) findViewById(R.id.gridviewImage);
        myAdapterImage = new MyAdapterImage(this, imgDaily);
        gridviewImage.setAdapter(myAdapterImage);

        // Récupération du numéro
        recordedNumber = (TextView) findViewById(R.id.recordedNumber);
        sendNumber = recordedNumber.getText().toString();

        // Arrêt de l'alarme
        stopAlarm = (Button) findViewById(R.id.stopAlarm);
        stopAlarm.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                text = number.getText().toString();
                stopAlarme();
            }
        });

        //Date
        SimpleDateFormat formatter = new SimpleDateFormat("EEEE dd MMMM yyyy");
        String currentDateString = formatter.format(new Date());
        Date = (TextView) findViewById(R.id.date);
        Date.setText(currentDateString);
    }

    @Override
    protected void onStart() {
        super.onStart();
        registerReceiver(scanModeReceiver, new IntentFilter(BluetoothAdapter.ACTION_SCAN_MODE_CHANGED));
        registerReceiver(bluetoothStateReceiver, new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED));
        registerReceiver(rfduinoReceiver, RFduinoService.getIntentFilter());

        updateState(bluetoothAdapter.isEnabled() ? STATE_DISCONNECTED : STATE_BLUETOOTH_OFF);
    }

    @Override
    protected void onStop() {
        super.onStop();

        bluetoothAdapter.stopLeScan(this);

        unregisterReceiver(scanModeReceiver);
        unregisterReceiver(bluetoothStateReceiver);
        unregisterReceiver(rfduinoReceiver);
    }

    @Override
    protected void onResume() {
        super.onResume();

        registerReceiver(scanModeReceiver, new IntentFilter(BluetoothAdapter.ACTION_SCAN_MODE_CHANGED));
        registerReceiver(bluetoothStateReceiver, new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED));
        registerReceiver(rfduinoReceiver, RFduinoService.getIntentFilter());

        updateState(bluetoothAdapter.isEnabled() ? STATE_DISCONNECTED : STATE_BLUETOOTH_OFF);
    }

    private void upgradeState(int newState) {
        if (newState > state) {
            updateState(newState);
        }
    }

    private void downgradeState(int newState) {
        if (newState < state) {
            updateState(newState);
        }
    }

    private void updateState(int newState) {
        state = newState;
        updateUi();
    }

    private void updateUi() {
        // Enable Bluetooth
        boolean on = state > STATE_BLUETOOTH_OFF;
        enableBluetoothButton.setEnabled(!on);
        enableBluetoothButton.setText(on ? "Bluetooth enabled" : "Enable Bluetooth");
        scanButton.setEnabled(on);

        // Scan
        if (scanStarted && scanning) {
            scanStatusText.setText("Scanning...");
            scanButton.setText("Stop Scan");
            scanButton.setEnabled(true);
        } else if (scanStarted) {
            scanStatusText.setText("Scan started...");
            scanButton.setEnabled(false);
        } else {
            scanStatusText.setText("");
            scanButton.setText("Scan");
            scanButton.setEnabled(true);
        }

        // Connect
        boolean connected = false;
        String connectionText = "Disconnected";
        if (state == STATE_CONNECTING) {
            connectionText = "Connecting...";
        } else if (state == STATE_CONNECTED) {
            connected = true;
            connectionText = "Connected";
            Log.d(TAG, "updateUI State connected");
        }
        connectionStatusText.setText(connectionText);
        connectButton.setEnabled(bluetoothDevice != null && state == STATE_DISCONNECTED);
    }


    @Override
    public void onLeScan(BluetoothDevice device, final int rssi, final byte[] scanRecord) {
        bluetoothAdapter.stopLeScan(this);
        bluetoothDevice = device;

        MainActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                deviceInfoText.setText(
                        Bluetooth.getDeviceInfoText(bluetoothDevice, rssi, scanRecord));
                updateUi();
            }
        });
    }


}
